@@ -24,6 +24,8 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " % d   w a r n i n g " ]   =   " % d   w a r n i n g " ; 
  l o c a l i z e d S t r i n g s [ " % d   w a r n i n g s " ]   =   " % d   w a r n i n g s " ; 
  l o c a l i z e d S t r i n g s [ " % d      % d " ]   =   " % 1 $ d      % 2 $ d " ; 
+ l o c a l i z e d S t r i n g s [ " % d      % d   p i x e l s " ]   =   " % 1 $ d      % 2 $ d   p i x e l s " ; 
+ l o c a l i z e d S t r i n g s [ " % d      % d   p i x e l s   ( N a t u r a l :   % d      % d   p i x e l s ) " ]   =   " % 1 $ d      % 2 $ d   ( N a t u r a l :   % 1 $ d      % 2 $ d   p i x e l s ) " ; 
  l o c a l i z e d S t r i n g s [ " % s   ( f r o m   c a c h e ) " ]   =   " % s   ( f r o m   c a c h e ) " ; 
  l o c a l i z e d S t r i n g s [ " % s   d o w n l o a d " ]   =   " % s   d o w n l o a d " ; 
  l o c a l i z e d S t r i n g s [ " % s   l a t e n c y " ]   =   " % s   l a t e n c y " ; 
@@ -34,20 +36,24 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " ( t e x t ) " ]   =   " ( t e x t ) " ; 
  l o c a l i z e d S t r i n g s [ " ( w h i t e s p a c e ) " ]   =   " ( w h i t e s p a c e ) " ; 
  l o c a l i z e d S t r i n g s [ " 1   m a t c h " ]   =   " 1   m a t c h " ; 
+ l o c a l i z e d S t r i n g s [ " A d d   a t t r i b u t e " ]   =   " A d d   a t t r i b u t e " ; 
  l o c a l i z e d S t r i n g s [ " A l l " ]   =   " A l l " ; 
  l o c a l i z e d S t r i n g s [ " A l w a y s   e n a b l e " ]   =   " A l w a y s   e n a b l e " ; 
  l o c a l i z e d S t r i n g s [ " A n   e r r o r   o c c u r r e d   t r y i n g   t o \ n r e a d   t h e    % s    t a b l e . " ]   =   " A n   e r r o r   o c c u r r e d   t r y i n g   t o \ n r e a d   t h e    % s    t a b l e . " ; 
  l o c a l i z e d S t r i n g s [ " A n   u n e x p e c t e d   e r r o r   % s   o c c u r r e d . " ]   =   " A n   u n e x p e c t e d   e r r o r   % s   o c c u r r e d . " ; 
+ l o c a l i z e d S t r i n g s [ " A s s e r t i o n   f a i l e d :   " ]   =   " A s s e r t i o n   f a i l e d :   " ; 
  l o c a l i z e d S t r i n g s [ " A v e r a g e " ]   =   " A v e r a g e " ; 
  l o c a l i z e d S t r i n g s [ " B r e a k p o i n t s " ]   =   " B r e a k p o i n t s " ; 
  l o c a l i z e d S t r i n g s [ " C O O K I E S " ]   =   " C O O K I E S " ; 
  l o c a l i z e d S t r i n g s [ " C a l l   S t a c k " ]   =   " C a l l   S t a c k " ; 
  l o c a l i z e d S t r i n g s [ " C a l l s " ]   =   " C a l l s " ; 
  l o c a l i z e d S t r i n g s [ " C l e a r   c h a n g e s   l o g . " ]   =   " C l e a r   c h a n g e s   l o g . " ; 
+ l o c a l i z e d S t r i n g s [ " C l e a r   C o n s o l e " ]   =   " C l e a r   C o n s o l e " ; 
  l o c a l i z e d S t r i n g s [ " C l e a r   c o n s o l e   l o g . " ]   =   " C l e a r   c o n s o l e   l o g . " ; 
  l o c a l i z e d S t r i n g s [ " C l o s u r e " ]   =   " C l o s u r e " ; 
  l o c a l i z e d S t r i n g s [ " C o m p u t e d   S t y l e " ]   =   " C o m p u t e d   S t y l e " ; 
  l o c a l i z e d S t r i n g s [ " C o o k i e s " ]   =   " C o o k i e s " ; 
+ l o c a l i z e d S t r i n g s [ " C o p y " ]   =   " C o p y " ; 
  l o c a l i z e d S t r i n g s [ " D A T A B A S E S " ]   =   " D A T A B A S E S " ; 
  l o c a l i z e d S t r i n g s [ " D O M " ]   =   " D O M " ; 
  l o c a l i z e d S t r i n g s [ " D a t a b a s e   n o   l o n g e r   h a s   e x p e c t e d   v e r s i o n . " ]   =   " D a t a b a s e   n o   l o n g e r   h a s   e x p e c t e d   v e r s i o n . " ; 
@@ -59,9 +65,10 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " D o c k   t o   m a i n   w i n d o w . " ]   =   " D o c k   t o   m a i n   w i n d o w . " ; 
  l o c a l i z e d S t r i n g s [ " D o c u m e n t s " ]   =   " D o c u m e n t s " ; 
  l o c a l i z e d S t r i n g s [ " D o m a i n " ]   =   " D o m a i n " ; 
- l o c a l i z e d S t r i n g s [ " D o n ' t   p a u s e   o n   e x c e p t i o n s . " ]   =   " D o n  t   p a u s e   o n   e x c e p t i o n s . " ; 
+ l o c a l i z e d S t r i n g s [ " D o n ' t   p a u s e   o n   e x c e p t i o n s . \ n C l i c k   t o   P a u s e   o n   a l l   e x c e p t i o n s . " ]   =   " D o n ' t   p a u s e   o n   e x c e p t i o n s . \ n C l i c k   t o   P a u s e   o n   a l l   e x c e p t i o n s . " ; 
  l o c a l i z e d S t r i n g s [ " D o u b l e - C l i c k   t o   A d d " ]   =   " D o u b l e - C l i c k   t o   A d d " ; 
  l o c a l i z e d S t r i n g s [ " D o u b l e - C l i c k   t o   t o g g l e   b e t w e e n   U R L   e n c o d e d   a n d   d e c o d e d   f o r m a t s " ]   =   " D o u b l e - C l i c k   t o   t o g g l e   b e t w e e n   U R L   e n c o d e d   a n d   d e c o d e d   f o r m a t s " ; 
+ l o c a l i z e d S t r i n g s [ " E d i t   a s   H T M L " ]   =   " E d i t   a s   H T M L " ; 
  l o c a l i z e d S t r i n g s [ " E l e m e n t s " ]   =   " E l e m e n t s " ; 
  l o c a l i z e d S t r i n g s [ " E n a b l e   D e b u g g i n g " ]   =   " E n a b l e   D e b u g g i n g " ; 
  l o c a l i z e d S t r i n g s [ " E n a b l e   P r o f i l i n g " ]   =   " E n a b l e   P r o f i l i n g " ; 
@@ -100,7 +107,8 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " O n l y   e n a b l e   f o r   t h i s   s e s s i o n " ]   =   " O n l y   e n a b l e   f o r   t h i s   s e s s i o n " ; 
  l o c a l i z e d S t r i n g s [ " O t h e r " ]   =   " O t h e r " ; 
  l o c a l i z e d S t r i n g s [ " P a t h " ]   =   " P a t h " ; 
- l o c a l i z e d S t r i n g s [ " P a u s e   o n   e x c e p t i o n s . " ]   =   " P a u s e   o n   e x c e p t i o n s . " ; 
+ l o c a l i z e d S t r i n g s [ " P a u s e   o n   a l l   e x c e p t i o n s . \ n C l i c k   t o   P a u s e   o n   u n c a u g h t   e x c e p t i o n s . " ]   =   " P a u s e   o n   a l l   e x c e p t i o n s . \ n C l i c k   t o   P a u s e   o n   u n c a u g h t   e x c e p t i o n s . " ; 
+ l o c a l i z e d S t r i n g s [ " P a u s e   o n   u n c a u g h t   e x c e p t i o n s . \ n C l i c k   t o   N o t   p a u s e   o n   e x c e p t i o n s . " ]   =   " P a u s e   o n   u n c a u g h t   e x c e p t i o n s . \ n C l i c k   t o   N o t   p a u s e   o n   e x c e p t i o n s . " ; 
  l o c a l i z e d S t r i n g s [ " P a u s e   s c r i p t   e x e c u t i o n . " ]   =   " P a u s e   s c r i p t   e x e c u t i o n . " ; 
  l o c a l i z e d S t r i n g s [ " P a u s e d " ]   =   " P a u s e d " ; 
  l o c a l i z e d S t r i n g s [ " P a u s i n g " ]   =   " P a u s i n g " ; 
@@ -158,7 +166,6 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " S t y l e s h e e t s " ]   =   " S t y l e s h e e t s " ; 
  l o c a l i z e d S t r i n g s [ " T h e    % s  \ n t a b l e   i s   e m p t y . " ]   =   " T h e    % s  \ n t a b l e   i s   e m p t y . " ; 
  l o c a l i z e d S t r i n g s [ " T h i s   s i t e   h a s   n o   c o o k i e s . " ]   =   " T h i s   s i t e   h a s   n o   c o o k i e s . " ; 
- l o c a l i z e d S t r i n g s [ " T h i s   s t o r a g e   i s   e m p t y . " ]   =   " T h i s   s t o r a g e   i s   e m p t y . " ; 
  l o c a l i z e d S t r i n g s [ " T i m e " ]   =   " T i m e " ; 
  l o c a l i z e d S t r i n g s [ " T o t a l " ]   =   " T o t a l " ; 
  l o c a l i z e d S t r i n g s [ " T r e e   ( T o p   D o w n ) " ]   =   " T r e e   ( T o p   D o w n ) " ; 
@@ -210,14 +217,60 @@ v a r   l o c a l i z e d S t r i n g s   =   n e w   O b j e c t ; 
  l o c a l i z e d S t r i n g s [ " A l l   N o d e s " ]   =   " A l l   N o d e s " ; 
  l o c a l i z e d S t r i n g s [ " L o a d   e v e n t   f i r e d " ]   =   " L o a d   e v e n t   f i r e d " ; 
  l o c a l i z e d S t r i n g s [ " D O M C o n t e n t   e v e n t   f i r e d " ]   =   " D O M C o n t e n t   e v e n t   f i r e d " ; 
- 
  l o c a l i z e d S t r i n g s [ " L o c a l   F i l e s " ]   =   " L o c a l   F i l e s " ; 
  l o c a l i z e d S t r i n g s [ " T i m e l i n e " ]   =   " T i m e l i n e " ; 
+ l o c a l i z e d S t r i n g s [ " L o a d i n g " ]   =   " L o a d i n g " ; 
+ l o c a l i z e d S t r i n g s [ " S c r i p t i n g " ]   =   " S c r i p t i n g " ; 
+ l o c a l i z e d S t r i n g s [ " R e n d e r i n g " ]   =   " R e n d e r i n g " ; 
  l o c a l i z e d S t r i n g s [ " R E C O R D S " ]   =   " R E C O R D S " ; 
- l o c a l i z e d S t r i n g s [ " D O M   E v e n t " ]   =   " D O M   E v e n t " ; 
+ l o c a l i z e d S t r i n g s [ " T I M E L I N E S " ]   =   " T I M E L I N E S " ; 
+ l o c a l i z e d S t r i n g s [ " E v e n t " ]   =   " E v e n t " ; 
  l o c a l i z e d S t r i n g s [ " L a y o u t " ]   =   " L a y o u t " ; 
  l o c a l i z e d S t r i n g s [ " R e c a l c u l a t e   S t y l e " ]   =   " R e c a l c u l a t e   S t y l e " ; 
  l o c a l i z e d S t r i n g s [ " P a i n t " ]   =   " P a i n t " ; 
- l o c a l i z e d S t r i n g s [ " L a y o u t " ]   =   " L a y o u t " ; 
  l o c a l i z e d S t r i n g s [ " P a r s e " ]   =   " P a r s e " ; 
- 
\ No newline at end of file
+ l o c a l i z e d S t r i n g s [ " I n s t a l l   T i m e r " ]   =   " I n s t a l l   T i m e r " ; 
+ l o c a l i z e d S t r i n g s [ " R e m o v e   T i m e r " ]   =   " R e m o v e   T i m e r " ; 
+ l o c a l i z e d S t r i n g s [ " T i m e r   F i r e d " ]   =   " T i m e r   F i r e d " ; 
+ l o c a l i z e d S t r i n g s [ " E v a l u a t e   S c r i p t " ]   =   " E v a l u a t e   S c r i p t " ; 
+ l o c a l i z e d S t r i n g s [ " X H R   L o a d " ]   =   " X H R   L o a d " ; 
+ l o c a l i z e d S t r i n g s [ " X H R   R e a d y   S t a t e   C h a n g e " ]   =   " X H R   R e a d y   S t a t e   C h a n g e " ; 
+ l o c a l i z e d S t r i n g s [ " M a r k " ]   =   " M a r k " ; 
+ l o c a l i z e d S t r i n g s [ " S e n d   R e q u e s t " ]   =   " S e n d   R e q u e s t " ; 
+ l o c a l i z e d S t r i n g s [ " R e c e i v e   R e s p o n s e " ]   =   " R e c e i v e   R e s p o n s e " ; 
+ l o c a l i z e d S t r i n g s [ " F i n i s h   L o a d i n g " ]   =   " F i n i s h   L o a d i n g " ; 
+ l o c a l i z e d S t r i n g s [ " C o n s o l e " ]   =   " C o n s o l e " ; 
+ l o c a l i z e d S t r i n g s [ " E d i t   T e x t " ]   =   " E d i t   T e x t " ; 
+ l o c a l i z e d S t r i n g s [ " D e l e t e   N o d e " ]   =   " D e l e t e   N o d e " ; 
+ l o c a l i z e d S t r i n g s [ " A d d   A t t r i b u t e " ]   =   " A d d   A t t r i b u t e " ; 
+ l o c a l i z e d S t r i n g s [ " E d i t   A t t r i b u t e " ]   =   " E d i t   A t t r i b u t e " ; 
+ l o c a l i z e d S t r i n g s [ " E d i t " ]   =   " E d i t " ; 
+ l o c a l i z e d S t r i n g s [ " A d d   N e w " ]   =   " A d d   N e w " ; 
+ l o c a l i z e d S t r i n g s [ " R e q u e s t   U R L " ]   =   " R e q u e s t   U R L " ; 
+ l o c a l i z e d S t r i n g s [ " % d   c o n s o l e   m e s s a g e s   a r e   n o t   s h o w n . " ]   =   " % d   c o n s o l e   m e s s a g e s   a r e   n o t   s h o w n . " ; 
+ l o c a l i z e d S t r i n g s [ " E d i t   a s   H T M L " ]   =   " E d i t   a s   H T M L " ; 
+ l o c a l i z e d S t r i n g s [ " C o p y   a s   H T M L " ]   =   " C o p y   a s   H T M L " ; 
+ l o c a l i z e d S t r i n g s [ " A u d i t s " ]   =   " A u d i t s " ; 
+ l o c a l i z e d S t r i n g s [ " R E S U L T S " ]   =   " R E S U L T S " ; 
+ l o c a l i z e d S t r i n g s [ " C l e a r   a u d i t   r e s u l t s . " ]   =   " C l e a r   a u d i t   r e s u l t s . " ; 
+ l o c a l i z e d S t r i n g s [ " S e l e c t   a u d i t s   t o   r u n " ]   =   " S e l e c t   a u d i t s   t o   r u n " ; 
+ l o c a l i z e d S t r i n g s [ " S e l e c t   A l l " ]   =   " S e l e c t   A l l " ; 
+ l o c a l i z e d S t r i n g s [ " R u n " ]   =   " R u n " ; 
+ l o c a l i z e d S t r i n g s [ " R u n n i n g . . . " ]   =   " R u n n i n g . . . " ; 
+ l o c a l i z e d S t r i n g s [ " A u d i t   P r e s e n t   S t a t e " ]   =   " A u d i t   P r e s e n t   S t a t e " ; 
+ l o c a l i z e d S t r i n g s [ " A u d i t   P r e s e n t   S t a t e   ( R e s o u r c e   T r a c k i n g   m u s t   b e   e n a b l e d ) " ]   =   " A u d i t   P r e s e n t   S t a t e   ( R e s o u r c e   T r a c k i n g   m u s t   b e   e n a b l e d ) " ; 
+ l o c a l i z e d S t r i n g s [ " N o   a u d i t s   t o   r u n " ]   =   " N o   a u d i t s   t o   r u n " ; 
+ l o c a l i z e d S t r i n g s [ " A d d   B r e a k p o i n t " ]   =   " A d d   B r e a k p o i n t " ; 
+ l o c a l i z e d S t r i n g s [ " A d d   C o n d i t i o n a l   B r e a k p o i n t . . . " ]   =   " A d d   C o n d i t i o n a l   B r e a k p o i n t . . . " ; 
+ l o c a l i z e d S t r i n g s [ " R e m o v e   B r e a k p o i n t " ]   =   " R e m o v e   B r e a k p o i n t " ; 
+ l o c a l i z e d S t r i n g s [ " E d i t   B r e a k p o i n t . . . " ]   =   " E d i t   B r e a k p o i n t . . . " ; 
+ l o c a l i z e d S t r i n g s [ " E n a b l e   B r e a k p o i n t " ]   =   " E n a b l e   B r e a k p o i n t " ; 
+ l o c a l i z e d S t r i n g s [ " D i s a b l e   B r e a k p o i n t " ]   =   " D i s a b l e   B r e a k p o i n t " ; 
+ l o c a l i z e d S t r i n g s [ " S t a r t   C P U   p r o f i l i n g   b y   p r e s s i n g < b r > t h e   % s   b u t t o n   o n   t h e   s t a t u s   b a r . " ]   =   " S t a r t   C P U   p r o f i l i n g   b y   p r e s s i n g < b r > t h e   % s   b u t t o n   o n   t h e   s t a t u s   b a r . " ; 
+ l o c a l i z e d S t r i n g s [ " W e l c o m e   t o   t h e   P r o f i l e s   p a n e l " ]   =   " W e l c o m e   t o   t h e   P r o f i l e s   p a n e l " ; 
+ l o c a l i z e d S t r i n g s [ " H e a d e r s " ]   =   " H e a d e r s " ; 
+ l o c a l i z e d S t r i n g s [ " C o n t e n t " ]   =   " C o n t e n t " ; 
+ l o c a l i z e d S t r i n g s [ " % d   c o o k i e s   ( % s ) " ]   =   " % d   c o o k i e s   ( % s ) " ; 
+ l o c a l i z e d S t r i n g s [ " S h o w   A l l   N o d e s   ( % d   M o r e ) " ]   =   " S h o w   A l l   N o d e s   ( % d   M o r e ) " ; 
+ l o c a l i z e d S t r i n g s [ " A c t i v a t e   a l l   b r e a k p o i n t s . " ]   =   " A c t i v a t e   a l l   b r e a k p o i n t s . " ; 
+ l o c a l i z e d S t r i n g s [ " D e a c t i v a t e   a l l   b r e a k p o i n t s . " ]   =   " D e a c t i v a t e   a l l   b r e a k p o i n t s . " ; 
\ No newline at end of file
