@@ -46,12 +46,6 @@
 #include "SerializedScriptValue.h"
 #include <wtf/OwnPtr.h>
 
-#if ENABLE(JAVASCRIPT_DEBUGGER) && USE(JSC)
-#include <parser/SourceCode.h>
-#include <runtime/JSValue.h>
-#include <runtime/UString.h>
-#endif
-
 namespace WebCore {
 
 InspectorFrontend::InspectorFrontend(InspectorController* inspectorController, ScriptObject webInspector)
@@ -250,7 +244,7 @@ void InspectorFrontend::addRecordToTimeline(const ScriptObject& record)
     function.call();
 }
 
-#if ENABLE(JAVASCRIPT_DEBUGGER) && USE(JSC)
+#if ENABLE(JAVASCRIPT_DEBUGGER)
 void InspectorFrontend::attachDebuggerWhenShown()
 {
     callSimpleFunction("attachDebuggerWhenShown");
@@ -266,6 +260,55 @@ void InspectorFrontend::debuggerWasDisabled()
     callSimpleFunction("debuggerWasDisabled");
 }
 
+void InspectorFrontend::parsedScriptSource(const String& sourceID, const String& url, const String& data, int firstLine)
+{
+    ScriptFunctionCall function(m_webInspector, "dispatch"); 
+    function.appendArgument("parsedScriptSource");
+    function.appendArgument(sourceID);
+    function.appendArgument(url);
+    function.appendArgument(data);
+    function.appendArgument(firstLine);
+    function.call();
+}
+
+void InspectorFrontend::restoredBreakpoint(const String& sourceID, const String& url, int line, bool enabled, const String& condition)
+{
+    ScriptFunctionCall function(m_webInspector, "dispatch");
+    function.appendArgument("restoredBreakpoint");
+    function.appendArgument(sourceID);
+    function.appendArgument(url);
+    function.appendArgument(line);
+    function.appendArgument(enabled);
+    function.appendArgument(condition);
+    function.call();
+}
+
+void InspectorFrontend::failedToParseScriptSource(const String& url, const String& data, int firstLine, int errorLine, const String& errorMessage)
+{
+    ScriptFunctionCall function(m_webInspector, "dispatch"); 
+    function.appendArgument("failedToParseScriptSource");
+    function.appendArgument(url);
+    function.appendArgument(data);
+    function.appendArgument(firstLine);
+    function.appendArgument(errorLine);
+    function.appendArgument(errorMessage);
+    function.call();
+}
+
+void InspectorFrontend::pausedScript(SerializedScriptValue* callFrames)
+{
+    ScriptValue callFramesValue = ScriptValue::deserialize(scriptState(), callFrames);
+    ScriptFunctionCall function(m_webInspector, "dispatch");
+    function.appendArgument("pausedScript");
+    function.appendArgument(callFramesValue);
+    function.call();
+}
+
+void InspectorFrontend::resumedScript()
+{
+    callSimpleFunction("resumedScript");
+}
+
 void InspectorFrontend::profilerWasEnabled()
 {
     callSimpleFunction("profilerWasEnabled");
@@ -276,29 +319,6 @@ void InspectorFrontend::profilerWasDisabled()
     callSimpleFunction("profilerWasDisabled");
 }
 
-void InspectorFrontend::parsedScriptSource(const JSC::SourceCode& source)
-{
-    ScriptFunctionCall function(m_webInspector, "dispatch"); 
-    function.appendArgument("parsedScriptSource");
-    function.appendArgument(JSC::UString(JSC::UString::from(source.provider()->asID())));
-    function.appendArgument(source.provider()->url());
-    function.appendArgument(JSC::UString(source.data(), source.length()));
-    function.appendArgument(source.firstLine());
-    function.call();
-}
-
-void InspectorFrontend::failedToParseScriptSource(const JSC::SourceCode& source, int errorLine, const JSC::UString& errorMessage)
-{
-    ScriptFunctionCall function(m_webInspector, "dispatch"); 
-    function.appendArgument("failedToParseScriptSource");
-    function.appendArgument(source.provider()->url());
-    function.appendArgument(JSC::UString(source.data(), source.length()));
-    function.appendArgument(source.firstLine());
-    function.appendArgument(errorLine);
-    function.appendArgument(errorMessage);
-    function.call();
-}
-
 void InspectorFrontend::addProfileHeader(const ScriptValue& profile)
 {
     ScriptFunctionCall function(m_webInspector, "dispatch"); 
@@ -332,20 +352,6 @@ void InspectorFrontend::didGetProfile(int callId, const ScriptValue& profile)
     function.appendArgument(profile);
     function.call();
 }
-
-void InspectorFrontend::pausedScript(SerializedScriptValue* callFrames)
-{
-    ScriptValue callFramesValue = ScriptValue::deserialize(scriptState(), callFrames);
-    ScriptFunctionCall function(m_webInspector, "dispatch"); 
-    function.appendArgument("pausedScript");
-    function.appendArgument(callFramesValue);
-    function.call();
-}
-
-void InspectorFrontend::resumedScript()
-{
-    callSimpleFunction("resumedScript");
-}
 #endif
 
 void InspectorFrontend::setDocument(const ScriptObject& root)
