@@ -468,17 +468,8 @@ WebInspector.loaded = function()
     var previousToolbarItem = toolbarElement.children[0];
 
     this.panelOrder = [];
-    for (var panelName in this.panels) {
-        var panel = this.panels[panelName];
-        var panelToolbarItem = panel.toolbarItem;
-        this.panelOrder.push(panel);
-        panelToolbarItem.addEventListener("click", this._toolbarItemClicked.bind(this));
-        if (previousToolbarItem)
-            toolbarElement.insertBefore(panelToolbarItem, previousToolbarItem.nextSibling);
-        else
-            toolbarElement.insertBefore(panelToolbarItem, toolbarElement.firstChild);
-        previousToolbarItem = panelToolbarItem;
-    }
+    for (var panelName in this.panels)
+        previousToolbarItem = WebInspector.addPanelToolbarIcon(toolbarElement, this.panels[panelName], previousToolbarItem);
 
     this.Tips = {
         ResourceNotCompressed: {id: 0, message: WebInspector.UIString("You could save bandwidth by having your web server compress this transfer with gzip or zlib.")}
@@ -529,6 +520,18 @@ WebInspector.loaded = function()
     InspectorFrontendHost.loaded();
 }
 
+WebInspector.addPanelToolbarIcon = function(toolbarElement, panel, previousToolbarItem)
+{
+    var panelToolbarItem = panel.toolbarItem;
+    this.panelOrder.push(panel);
+    panelToolbarItem.addEventListener("click", this._toolbarItemClicked.bind(this));
+    if (previousToolbarItem)
+        toolbarElement.insertBefore(panelToolbarItem, previousToolbarItem.nextSibling);
+    else
+        toolbarElement.insertBefore(panelToolbarItem, toolbarElement.firstChild);
+    return panelToolbarItem;
+}
+
 var windowLoaded = function()
 {
     var localizedStringsURL = InspectorFrontendHost.localizedStringsURL();
@@ -683,9 +686,7 @@ WebInspector.documentKeyDown = function(event)
                 var isFindKey = event.ctrlKey && !event.metaKey && !event.altKey && !event.shiftKey;
 
             if (isFindKey) {
-                var searchField = document.getElementById("search");
-                searchField.focus();
-                searchField.select();
+                WebInspector.focusSearchField();
                 event.preventDefault();
             }
 
@@ -749,8 +750,11 @@ WebInspector.documentKeyDown = function(event)
                 var shouldShowAuditsPanel = event.ctrlKey && !event.shiftKey && !event.metaKey && event.altKey;
 
             if (shouldShowAuditsPanel) {
-                if (!this.panels.audits)
+                if (!this.panels.audits) {
                     this.panels.audits = new WebInspector.AuditsPanel();
+                    var toolbarElement = document.getElementById("toolbar");
+                    WebInspector.addPanelToolbarIcon(toolbarElement, this.panels.audits, this.panels.console.toolbarItem);
+                }
                 this.currentPanel = this.panels.audits;
             }
 
@@ -872,6 +876,13 @@ WebInspector.updateSearchLabel = function()
     }
 }
 
+WebInspector.focusSearchField = function()
+{
+    var searchField = document.getElementById("search");
+    searchField.focus();
+    searchField.select();
+}
+
 WebInspector.toggleAttach = function()
 {
     this.attached = !this.attached;
@@ -1201,6 +1212,13 @@ WebInspector.parsedScriptSource = function(sourceID, sourceURL, source, starting
     this.panels.scripts.addScript(sourceID, sourceURL, source, startingLine);
 }
 
+WebInspector.restoredBreakpoint = function(sourceID, sourceURL, line, enabled, condition)
+{
+    var breakpoint = new WebInspector.Breakpoint(sourceURL, line, sourceID, condition);
+    breakpoint.enabled = enabled;
+    this.panels.scripts.addBreakpoint(breakpoint);
+}
+
 WebInspector.failedToParseScriptSource = function(sourceURL, source, startingLine, errorLine, errorMessage)
 {
     this.panels.scripts.addScript(null, sourceURL, source, startingLine, errorLine, errorMessage);
@@ -1331,7 +1349,7 @@ WebInspector.log = function(message)
         WebInspector.log.repeatCount = repeatCount;
 
         // ConsoleMessage expects a proxy object
-        message = new WebInspector.ObjectProxy(null, null, [], 0, message, false);
+        message = new WebInspector.ObjectProxy(null, null, [], message, false);
 
         // post the message
         var msg = new WebInspector.ConsoleMessage(
