@@ -445,7 +445,9 @@ private:
     bool shouldTreatURLAsSameAsCurrent(const KURL&) const;
 
     void updateSandboxFlags();
-    
+    // FIXME: isDocumentSandboxed should eventually replace isSandboxed.
+    bool isDocumentSandboxed(SandboxFlags) const;
+
     Frame* m_frame;
     FrameLoaderClient* m_client;
 
