@@ -29,122 +29,56 @@
 
 namespace WebCore {
 
-class GIFImageDecoderPrivate {
-public:
-    GIFImageDecoderPrivate(GIFImageDecoder* decoder = 0)
-        : m_reader(decoder)
-        , m_readOffset(0)
-    {
-    }
-
-    ~GIFImageDecoderPrivate()
-    {
-        m_reader.close();
-    }
-
-    bool decode(SharedBuffer* data, 
-                GIFImageDecoder::GIFQuery query = GIFImageDecoder::GIFFullQuery,
-                unsigned int haltFrame = -1)
-    {
-        return m_reader.read((const unsigned char*)data->data() + m_readOffset, data->size() - m_readOffset, 
-                             query,
-                             haltFrame);
-    }
-
-    unsigned frameCount() const { return m_reader.images_count; }
-    int repetitionCount() const { return m_reader.loop_count; }
-
-    void setReadOffset(unsigned o) { m_readOffset = o; }
-
-    bool isTransparent() const { return m_reader.frame_reader->is_transparent; }
-
-    void getColorMap(unsigned char*& map, unsigned& size) const
-    {
-        if (m_reader.frame_reader->is_local_colormap_defined) {
-            map = m_reader.frame_reader->local_colormap;
-            size = (unsigned)m_reader.frame_reader->local_colormap_size;
-        } else {
-            map = m_reader.global_colormap;
-            size = m_reader.global_colormap_size;
-        }
-    }
-
-    unsigned frameXOffset() const { return m_reader.frame_reader->x_offset; }
-    unsigned frameYOffset() const { return m_reader.frame_reader->y_offset; }
-    unsigned frameWidth() const { return m_reader.frame_reader->width; }
-    unsigned frameHeight() const { return m_reader.frame_reader->height; }
-
-    int transparentPixel() const { return m_reader.frame_reader->tpixel; }
-
-    unsigned duration() const { return m_reader.frame_reader->delay_time; }
-
-private:
-    GIFImageReader m_reader;
-    unsigned m_readOffset;
-};
-
 GIFImageDecoder::GIFImageDecoder()
-    : m_frameCountValid(true)
+    : m_alreadyScannedThisDataForFrameCount(true)
     , m_repetitionCount(cAnimationLoopOnce)
-    , m_reader(0)
+    , m_readOffset(0)
 {
 }
 
 GIFImageDecoder::~GIFImageDecoder()
 {
-    delete m_reader;
 }
 
-// Take the data and store it.
 void GIFImageDecoder::setData(SharedBuffer* data, bool allDataReceived)
 {
     if (m_failed)
         return;
 
-    // Cache our new data.
     ImageDecoder::setData(data, allDataReceived);
 
-    // Our frame count is now unknown.
-    m_frameCountValid = false;
+    // We need to rescan the frame count, as the new data may have changed it.
+    m_alreadyScannedThisDataForFrameCount = false;
 
-    // Create the GIF reader.
     if (!m_reader && !m_failed)
-        m_reader = new GIFImageDecoderPrivate(this);
+        m_reader.set(new GIFImageReader(this));
 }
 
-// Whether or not the size information has been decoded yet.
 bool GIFImageDecoder::isSizeAvailable()
 {
     if (!ImageDecoder::isSizeAvailable() && !failed() && m_reader)
-         decode(GIFSizeQuery, 0);
+         decode(0, GIFSizeQuery);
 
     return ImageDecoder::isSizeAvailable();
 }
 
-// The total number of frames for the image.  Will scan the image data for the answer
-// (without necessarily decoding all of the individual frames).
 size_t GIFImageDecoder::frameCount()
 {
-    // If the decoder had an earlier error, we will just return what we had decoded
-    // so far.
-    if (!m_frameCountValid) {
-        // FIXME: Scanning all the data has O(n^2) behavior if the data were to come in really
-        // slowly.  Might be interesting to try to clone our existing read session to preserve
-        // state, but for now we just crawl all the data.  Note that this is no worse than what
-        // ImageIO does on Mac right now (it also crawls all the data again).
-        GIFImageDecoderPrivate reader;
-        // This function may fail, but we want to keep any partial data it may
-        // have decoded, so don't mark it is invalid. If there is an overflow
-        // or some serious error, m_failed will have gotten set for us.
-        reader.decode(m_data.get(), GIFFrameCountQuery);
-        m_frameCountValid = true;
-        m_frameBufferCache.resize(reader.frameCount());
+    if (!m_alreadyScannedThisDataForFrameCount) {
+        // FIXME: Scanning all the data has O(n^2) behavior if the data were to
+        // come in really slowly.  Might be interesting to try to clone our
+        // existing read session to preserve state, but for now we just crawl
+        // all the data.  Note that this is no worse than what ImageIO does on
+        // Mac right now (it also crawls all the data again).
+        GIFImageReader reader(0);
+        reader.read((const unsigned char*)m_data->data(), m_data->size(), GIFFrameCountQuery, static_cast<unsigned>(-1));
+        m_alreadyScannedThisDataForFrameCount = true;
+        m_frameBufferCache.resize(reader.images_count);
     }
 
     return m_frameBufferCache.size();
 }
 
-// The number of repetitions to perform for an animation loop.
 int GIFImageDecoder::repetitionCount() const
 {
     // This value can arrive at any point in the image data stream.  Most GIFs
@@ -161,7 +95,7 @@ int GIFImageDecoder::repetitionCount() const
         // cAnimationLoopOnce (-1) when its current incarnation hasn't actually
         // seen a loop count yet; in this case we return our previously-cached
         // value.
-        const int repetitionCount = m_reader->repetitionCount();
+        const int repetitionCount = m_reader->loop_count;
         if (repetitionCount != cLoopCountNotSeen)
             m_repetitionCount = repetitionCount;
     }
@@ -175,7 +109,7 @@ RGBA32Buffer* GIFImageDecoder::frameBufferAtIndex(size_t index)
 
     RGBA32Buffer& frame = m_frameBufferCache[index];
     if (frame.status() != RGBA32Buffer::FrameComplete && m_reader)
-        decode(GIFFullQuery, index + 1); // Decode this frame.
+        decode(index + 1, GIFFullQuery); // Decode this frame.
     return &frame;
 }
 
@@ -229,49 +163,165 @@ void GIFImageDecoder::clearFrameBufferCache(size_t clearBeforeFrame)
     }
 }
 
-// Feed data to the GIF reader.
-void GIFImageDecoder::decode(GIFQuery query, unsigned haltAtFrame)
-{
-    if (m_failed)
-        return;
-
-    m_failed = !m_reader->decode(m_data.get(), query, haltAtFrame);
-    
-    if (m_failed) {
-        delete m_reader;
-        m_reader = 0;
-    }
-}
-
-// Callbacks from the GIF reader.
 bool GIFImageDecoder::sizeNowAvailable(unsigned width, unsigned height)
 {
-    return setSize(width, height);
+    if (!setSize(width, height))
+        return false;
+
+    prepareScaleDataIfNecessary();
+    return true;
 }
 
 void GIFImageDecoder::decodingHalted(unsigned bytesLeft)
 {
-    m_reader->setReadOffset(m_data->size() - bytesLeft);
+    m_readOffset = m_data->size() - bytesLeft;
+}
+
+bool GIFImageDecoder::haveDecodedRow(unsigned frameIndex, unsigned char* rowBuffer, unsigned char* rowEnd, unsigned rowNumber, unsigned repeatCount, bool writeTransparentPixels)
+{
+    const GIFFrameReader* frameReader = m_reader->frame_reader;
+    // The pixel data and coordinates supplied to us are relative to the frame's
+    // origin within the entire image size, i.e.
+    // (frameReader->x_offset, frameReader->y_offset).  There is no guarantee
+    // that (rowEnd - rowBuffer) == (size().width() - frameReader->x_offset), so
+    // we must ensure we don't run off the end of either the source data or the
+    // row's X-coordinates.
+    int xBegin = upperBoundScaledX(frameReader->x_offset);
+    int yBegin = upperBoundScaledY(frameReader->y_offset + rowNumber);
+    int xEnd = lowerBoundScaledX(std::min(xBegin + static_cast<int>(rowEnd - rowBuffer), size().width()) - 1, xBegin + 1) + 1;
+    int yEnd = lowerBoundScaledY(std::min(yBegin + static_cast<int>(repeatCount), size().height()) - 1, yBegin + 1) + 1;
+    if (!rowBuffer || (xBegin < 0) || (yBegin < 0) || (xEnd <= xBegin) || (yEnd <= yBegin))
+        return true;
+
+    // Get the colormap.
+    const unsigned char* colorMap;
+    unsigned colorMapSize;
+    if (frameReader->is_local_colormap_defined) {
+        colorMap = frameReader->local_colormap;
+        colorMapSize = (unsigned)frameReader->local_colormap_size;
+    } else {
+        colorMap = m_reader->global_colormap;
+        colorMapSize = m_reader->global_colormap_size;
+    }
+    if (!colorMap)
+        return true;
+
+    // Initialize the frame if necessary.
+    RGBA32Buffer& buffer = m_frameBufferCache[frameIndex];
+    if ((buffer.status() == RGBA32Buffer::FrameEmpty) && !initFrameBuffer(frameIndex))
+        return false;
+
+    // Write one row's worth of data into the frame.  
+    for (int x = xBegin; x < xEnd; ++x) {
+        const unsigned char sourceValue = *(rowBuffer + (m_scaled ? m_scaledColumns[x] : x) - frameReader->x_offset);
+        if ((!frameReader->is_transparent || (sourceValue != frameReader->tpixel)) && (sourceValue < colorMapSize)) {
+            const size_t colorIndex = static_cast<size_t>(sourceValue) * 3;
+            buffer.setRGBA(x, yBegin, colorMap[colorIndex], colorMap[colorIndex + 1], colorMap[colorIndex + 2], 255);
+        } else {
+            m_currentBufferSawAlpha = true;
+            // We may or may not need to write transparent pixels to the buffer.
+            // If we're compositing against a previous image, it's wrong, and if
+            // we're writing atop a cleared, fully transparent buffer, it's
+            // unnecessary; but if we're decoding an interlaced gif and
+            // displaying it "Haeberli"-style, we must write these for passes
+            // beyond the first, or the initial passes will "show through" the
+            // later ones.
+            if (writeTransparentPixels)
+                buffer.setRGBA(x, yBegin, 0, 0, 0, 0);
+        }
+    }
+
+    // Tell the frame to copy the row data if need be.
+    if (repeatCount > 1)
+        buffer.copyRowNTimes(xBegin, xEnd, yBegin, yEnd);
+
+    return true;
+}
+
+void GIFImageDecoder::frameComplete(unsigned frameIndex, unsigned frameDuration, RGBA32Buffer::FrameDisposalMethod disposalMethod)
+{
+    // Initialize the frame if necessary.  Some GIFs insert do-nothing frames,
+    // in which case we never reach haveDecodedRow() before getting here.
+    RGBA32Buffer& buffer = m_frameBufferCache[frameIndex];
+    if ((buffer.status() == RGBA32Buffer::FrameEmpty) && !initFrameBuffer(frameIndex))
+        return;
+
+    buffer.setStatus(RGBA32Buffer::FrameComplete);
+    buffer.setDuration(frameDuration);
+    buffer.setDisposalMethod(disposalMethod);
+
+    if (!m_currentBufferSawAlpha) {
+        // The whole frame was non-transparent, so it's possible that the entire
+        // resulting buffer was non-transparent, and we can setHasAlpha(false).
+        if (buffer.rect().contains(IntRect(IntPoint(), scaledSize())))
+            buffer.setHasAlpha(false);
+        else if (frameIndex) {
+            // Tricky case.  This frame does not have alpha only if everywhere
+            // outside its rect doesn't have alpha.  To know whether this is
+            // true, we check the start state of the frame -- if it doesn't have
+            // alpha, we're safe.
+            //
+            // First skip over prior DisposeOverwritePrevious frames (since they
+            // don't affect the start state of this frame) the same way we do in
+            // initFrameBuffer().
+            const RGBA32Buffer* prevBuffer = &m_frameBufferCache[--frameIndex];
+            while (frameIndex && (prevBuffer->disposalMethod() == RGBA32Buffer::DisposeOverwritePrevious))
+                prevBuffer = &m_frameBufferCache[--frameIndex];
+
+            // Now, if we're at a DisposeNotSpecified or DisposeKeep frame, then
+            // we can say we have no alpha if that frame had no alpha.  But
+            // since in initFrameBuffer() we already copied that frame's alpha
+            // state into the current frame's, we need do nothing at all here.
+            //
+            // The only remaining case is a DisposeOverwriteBgcolor frame.  If
+            // it had no alpha, and its rect is contained in the current frame's
+            // rect, we know the current frame has no alpha.
+            if ((prevBuffer->disposalMethod() == RGBA32Buffer::DisposeOverwriteBgcolor) && !prevBuffer->hasAlpha() && buffer.rect().contains(prevBuffer->rect()))
+                buffer.setHasAlpha(false);
+        }
+    }
+}
+
+void GIFImageDecoder::gifComplete()
+{
+    if (m_reader)
+        m_repetitionCount = m_reader->loop_count;
+    m_reader.clear();
+}
+
+void GIFImageDecoder::decode(unsigned haltAtFrame, GIFQuery query)
+{
+    if (m_failed)
+        return;
+
+    m_failed = !m_reader->read((const unsigned char*)m_data->data() + m_readOffset, m_data->size() - m_readOffset, query, haltAtFrame);
+
+    if (m_failed)
+        m_reader.clear();
 }
 
 bool GIFImageDecoder::initFrameBuffer(unsigned frameIndex)
 {
     // Initialize the frame rect in our buffer.
-    IntRect frameRect(m_reader->frameXOffset(), m_reader->frameYOffset(),
-                      m_reader->frameWidth(), m_reader->frameHeight());
+    const GIFFrameReader* frameReader = m_reader->frame_reader;
+    IntRect frameRect(frameReader->x_offset, frameReader->y_offset, frameReader->width, frameReader->height);
 
-    // Make sure the frameRect doesn't extend past the bottom-right of the buffer.
+    // Make sure the frameRect doesn't extend outside the buffer.
     if (frameRect.right() > size().width())
-        frameRect.setWidth(size().width() - m_reader->frameXOffset());
+        frameRect.setWidth(size().width() - frameReader->x_offset);
     if (frameRect.bottom() > size().height())
-        frameRect.setHeight(size().height() - m_reader->frameYOffset());
+        frameRect.setHeight(size().height() - frameReader->y_offset);
 
     RGBA32Buffer* const buffer = &m_frameBufferCache[frameIndex];
-    buffer->setRect(frameRect);
+    int left = upperBoundScaledX(frameRect.x());
+    int right = lowerBoundScaledX(frameRect.right(), left);
+    int top = upperBoundScaledY(frameRect.y());
+    int bottom = lowerBoundScaledY(frameRect.bottom(), top);
+    buffer->setRect(IntRect(left, top, right - left, bottom - top));
     
-    if (frameIndex == 0) {
+    if (!frameIndex) {
         // This is the first frame, so we're not relying on any previous data.
-        if (!buffer->setSize(size().width(), size().height())) {
+        if (!buffer->setSize(scaledSize().width(), scaledSize().height())) {
             m_failed = true;
             return false;
         }
@@ -285,28 +335,25 @@ bool GIFImageDecoder::initFrameBuffer(unsigned frameIndex)
         // first frame specifies this method, it will get treated like
         // DisposeOverwriteBgcolor below and reset to a completely empty image.)
         const RGBA32Buffer* prevBuffer = &m_frameBufferCache[--frameIndex];
-        RGBA32Buffer::FrameDisposalMethod prevMethod =
-            prevBuffer->disposalMethod();
-        while ((frameIndex > 0)
-               && (prevMethod == RGBA32Buffer::DisposeOverwritePrevious)) {
+        RGBA32Buffer::FrameDisposalMethod prevMethod = prevBuffer->disposalMethod();
+        while (frameIndex && (prevMethod == RGBA32Buffer::DisposeOverwritePrevious)) {
             prevBuffer = &m_frameBufferCache[--frameIndex];
             prevMethod = prevBuffer->disposalMethod();
         }
         ASSERT(prevBuffer->status() == RGBA32Buffer::FrameComplete);
 
-        if ((prevMethod == RGBA32Buffer::DisposeNotSpecified) ||
-                (prevMethod == RGBA32Buffer::DisposeKeep)) {
+        if ((prevMethod == RGBA32Buffer::DisposeNotSpecified) || (prevMethod == RGBA32Buffer::DisposeKeep)) {
             // Preserve the last frame as the starting state for this frame.
             buffer->copyBitmapData(*prevBuffer);
         } else {
             // We want to clear the previous frame to transparent, without
             // affecting pixels in the image outside of the frame.
             const IntRect& prevRect = prevBuffer->rect();
-            if ((frameIndex == 0)
-                || prevRect.contains(IntRect(IntPoint(), size()))) {
+            const IntSize& bufferSize = scaledSize();
+            if (!frameIndex || prevRect.contains(IntRect(IntPoint(), scaledSize()))) {
                 // Clearing the first frame, or a frame the size of the whole
                 // image, results in a completely empty image.
-                if (!buffer->setSize(size().width(), size().height())) {
+                if (!buffer->setSize(bufferSize.width(), bufferSize.height())) {
                     m_failed = true;
                     return false;
                 }
@@ -331,115 +378,4 @@ bool GIFImageDecoder::initFrameBuffer(unsigned frameIndex)
     return true;
 }
 
-void GIFImageDecoder::haveDecodedRow(unsigned frameIndex,
-                                     unsigned char* rowBuffer,
-                                     unsigned char* rowEnd,
-                                     unsigned rowNumber,
-                                     unsigned repeatCount,
-                                     bool writeTransparentPixels)
-{
-    // The pixel data and coordinates supplied to us are relative to the frame's
-    // origin within the entire image size, i.e.
-    // (m_reader->frameXOffset(), m_reader->frameYOffset()).
-    int x = m_reader->frameXOffset();
-    const int y = m_reader->frameYOffset() + rowNumber;
-
-    // Sanity-check the arguments.
-    if ((rowBuffer == 0) || (y >= size().height()))
-        return;
-
-    // Get the colormap.
-    unsigned colorMapSize;
-    unsigned char* colorMap;
-    m_reader->getColorMap(colorMap, colorMapSize);
-    if (!colorMap)
-        return;
-
-    // Initialize the frame if necessary.
-    RGBA32Buffer& buffer = m_frameBufferCache[frameIndex];
-    if ((buffer.status() == RGBA32Buffer::FrameEmpty) && !initFrameBuffer(frameIndex))
-        return;
-
-    // Write one row's worth of data into the frame.  There is no guarantee that
-    // (rowEnd - rowBuffer) == (size().width() - m_reader->frameXOffset()), so
-    // we must ensure we don't run off the end of either the source data or the
-    // row's X-coordinates.
-    for (unsigned char* sourceAddr = rowBuffer; (sourceAddr != rowEnd) && (x < size().width()); ++sourceAddr, ++x) {
-        const unsigned char sourceValue = *sourceAddr;
-        if ((!m_reader->isTransparent() || (sourceValue != m_reader->transparentPixel())) && (sourceValue < colorMapSize)) {
-            const size_t colorIndex = static_cast<size_t>(sourceValue) * 3;
-            buffer.setRGBA(x, y, colorMap[colorIndex], colorMap[colorIndex + 1], colorMap[colorIndex + 2], 255);
-        } else {
-            m_currentBufferSawAlpha = true;
-            // We may or may not need to write transparent pixels to the buffer.
-            // If we're compositing against a previous image, it's wrong, and if
-            // we're writing atop a cleared, fully transparent buffer, it's
-            // unnecessary; but if we're decoding an interlaced gif and
-            // displaying it "Haeberli"-style, we must write these for passes
-            // beyond the first, or the initial passes will "show through" the
-            // later ones.
-            if (writeTransparentPixels)
-                buffer.setRGBA(x, y, 0, 0, 0, 0);
-        }
-    }
-
-    // Tell the frame to copy the row data if need be.
-    if (repeatCount > 1)
-        buffer.copyRowNTimes(m_reader->frameXOffset(), x, y, std::min(y + static_cast<int>(repeatCount), size().height()));
-}
-
-void GIFImageDecoder::frameComplete(unsigned frameIndex, unsigned frameDuration, RGBA32Buffer::FrameDisposalMethod disposalMethod)
-{
-    // Initialize the frame if necessary.  Some GIFs insert do-nothing frames,
-    // in which case we never reach haveDecodedRow() before getting here.
-    RGBA32Buffer& buffer = m_frameBufferCache[frameIndex];
-    if ((buffer.status() == RGBA32Buffer::FrameEmpty) && !initFrameBuffer(frameIndex))
-        return;
-
-    buffer.setStatus(RGBA32Buffer::FrameComplete);
-    buffer.setDuration(frameDuration);
-    buffer.setDisposalMethod(disposalMethod);
-
-    if (!m_currentBufferSawAlpha) {
-        // The whole frame was non-transparent, so it's possible that the entire
-        // resulting buffer was non-transparent, and we can setHasAlpha(false).
-        if (buffer.rect().contains(IntRect(IntPoint(), size())))
-            buffer.setHasAlpha(false);
-        else if (frameIndex > 0) {
-            // Tricky case.  This frame does not have alpha only if everywhere
-            // outside its rect doesn't have alpha.  To know whether this is
-            // true, we check the start state of the frame -- if it doesn't have
-            // alpha, we're safe.
-            //
-            // First skip over prior DisposeOverwritePrevious frames (since they
-            // don't affect the start state of this frame) the same way we do in
-            // initFrameBuffer().
-            const RGBA32Buffer* prevBuffer = &m_frameBufferCache[--frameIndex];
-            while ((frameIndex > 0)
-                   && (prevBuffer->disposalMethod() == RGBA32Buffer::DisposeOverwritePrevious))
-                prevBuffer = &m_frameBufferCache[--frameIndex];
-
-            // Now, if we're at a DisposeNotSpecified or DisposeKeep frame, then
-            // we can say we have no alpha if that frame had no alpha.  But
-            // since in initFrameBuffer() we already copied that frame's alpha
-            // state into the current frame's, we need do nothing at all here.
-            //
-            // The only remaining case is a DisposeOverwriteBgcolor frame.  If
-            // it had no alpha, and its rect is contained in the current frame's
-            // rect, we know the current frame has no alpha.
-            if ((prevBuffer->disposalMethod() == RGBA32Buffer::DisposeOverwriteBgcolor)
-                && !prevBuffer->hasAlpha() && buffer.rect().contains(prevBuffer->rect()))
-                buffer.setHasAlpha(false);
-        }
-    }
-}
-
-void GIFImageDecoder::gifComplete()
-{
-    if (m_reader)
-        m_repetitionCount = m_reader->repetitionCount();
-    delete m_reader;
-    m_reader = 0;
-}
-
 } // namespace WebCore
