@@ -28,6 +28,7 @@
 #define JPEGImageDecoder_h
 
 #include "ImageDecoder.h"
+#include <wtf/OwnPtr.h>
 
 namespace WebCore {
 
@@ -37,39 +38,25 @@ namespace WebCore {
     class JPEGImageDecoder : public ImageDecoder {
     public:
         JPEGImageDecoder();
-        ~JPEGImageDecoder();
+        virtual ~JPEGImageDecoder();
 
+        // ImageDecoder
         virtual String filenameExtension() const { return "jpg"; }
-
-        // Take the data and store it.
         virtual void setData(SharedBuffer* data, bool allDataReceived);
-
-        // Whether or not the size information has been decoded yet.
         virtual bool isSizeAvailable();
-
+        virtual bool setSize(unsigned width, unsigned height);
         virtual RGBA32Buffer* frameBufferAtIndex(size_t index);
-        
         virtual bool supportsAlpha() const { return false; }
 
-        void decode(bool sizeOnly = false);
-
-        JPEGImageReader* reader() { return m_reader; }
-
         bool outputScanlines();
         void jpegComplete();
 
-#if ENABLE(IMAGE_DECODER_DOWN_SAMPLING)
-        bool setSize(int width, int height)
-        {
-            if (!ImageDecoder::setSize(width, height))
-                return false;
-            prepareScaleDataIfNecessary();
-            return true;
-        }
-#endif
-
     private:
-        JPEGImageReader* m_reader;
+        // Decodes the image.  If |onlySize| is true, stops decoding after
+        // calculating the image size.
+        void decode(bool onlySize);
+
+        OwnPtr<JPEGImageReader> m_reader;
     };
 
 } // namespace WebCore
