@@ -27,6 +27,7 @@
 #define PNGImageDecoder_h
 
 #include "ImageDecoder.h"
+#include <wtf/OwnPtr.h>
 
 namespace WebCore {
 
@@ -36,22 +37,14 @@ namespace WebCore {
     class PNGImageDecoder : public ImageDecoder {
     public:
         PNGImageDecoder();
-        ~PNGImageDecoder();
+        virtual ~PNGImageDecoder();
 
+        // ImageDecoder
         virtual String filenameExtension() const { return "png"; }
-
-        // Take the data and store it.
         virtual void setData(SharedBuffer* data, bool allDataReceived);
-
-        // Whether or not the size information has been decoded yet.
         virtual bool isSizeAvailable();
-
         virtual RGBA32Buffer* frameBufferAtIndex(size_t index);
 
-        void decode(bool sizeOnly = false);
-
-        PNGImageReader* reader() { return m_reader; }
-
         // Callbacks from libpng
         void decodingFailed();
         void headerAvailable();
@@ -59,7 +52,11 @@ namespace WebCore {
         void pngComplete();
 
     private:
-        PNGImageReader* m_reader;
+        // Decodes the image.  If |onlySize| is true, stops decoding after
+        // calculating the image size.
+        void decode(bool onlySize);
+
+        OwnPtr<PNGImageReader> m_reader;
     };
 
 } // namespace WebCore
