@@ -30,7 +30,6 @@
 #define Database_h
 
 #if ENABLE(DATABASE)
-#include <wtf/MessageQueue.h>
 #include "PlatformString.h"
 #include "SecurityOrigin.h"
 #include "SQLiteDatabase.h"
@@ -41,7 +40,6 @@
 
 #include <wtf/Forward.h>
 #include <wtf/HashSet.h>
-#include <wtf/OwnPtr.h>
 #include <wtf/PassRefPtr.h>
 #include <wtf/RefPtr.h>
 #include <wtf/Deque.h>
@@ -54,12 +52,14 @@ namespace WebCore {
 
 class DatabaseAuthorizer;
 class DatabaseThread;
-class Document;
+class ScriptExecutionContext;
 class SQLResultSet;
 class SQLTransactionCallback;
+class SQLTransactionClient;
+class SQLTransactionCoordinator;
 class SQLTransactionErrorCallback;
 class SQLValue;
-    
+
 typedef int ExceptionCode;
 
 class Database : public ThreadSafeShared<Database> {
@@ -67,17 +67,20 @@ class Database : public ThreadSafeShared<Database> {
     friend class SQLStatement;
     friend class SQLTransaction;
 public:
+    static void setIsAvailable(bool);
+    static bool isAvailable();
+
     ~Database();
 
 // Direct support for the DOM API
-    static PassRefPtr<Database> openDatabase(Document* document, const String& name, const String& expectedVersion, const String& displayName, unsigned long estimatedSize, ExceptionCode&);
+    static PassRefPtr<Database> openDatabase(ScriptExecutionContext* context, const String& name, const String& expectedVersion, const String& displayName, unsigned long estimatedSize, ExceptionCode&);
     String version() const;
-    void changeVersion(const String& oldVersion, const String& newVersion, 
+    void changeVersion(const String& oldVersion, const String& newVersion,
                        PassRefPtr<SQLTransactionCallback> callback, PassRefPtr<SQLTransactionErrorCallback> errorCallback,
                        PassRefPtr<VoidCallback> successCallback);
     void transaction(PassRefPtr<SQLTransactionCallback> callback, PassRefPtr<SQLTransactionErrorCallback> errorCallback,
-                     PassRefPtr<VoidCallback> successCallback);
-    
+                     PassRefPtr<VoidCallback> successCallback, bool readOnly);
+
 // Internal engine support
     static const String& databaseInfoTableName();
 
@@ -87,10 +90,13 @@ public:
 
     Vector<String> tableNames();
 
-    Document* document() const { return m_document.get(); }
-    PassRefPtr<SecurityOrigin> securityOriginCopy() const;
+    ScriptExecutionContext* scriptExecutionContext() const { return m_scriptExecutionContext.get(); }
+    SecurityOrigin* securityOrigin() const;
     String stringIdentifier() const;
-    
+    String displayName() const;
+    unsigned long estimatedSize() const;
+    String fileName() const;
+
     bool getVersionFromDatabase(String&);
     bool setVersionInDatabase(const String&);
     void setExpectedVersion(const String&);
@@ -101,7 +107,7 @@ public:
 
     void close();
     bool opened() const { return m_opened; }
-    
+
     void stop();
     bool stopped() const { return m_stopped; }
 
@@ -116,30 +122,39 @@ public:
 
     Vector<String> performGetTableNames();
 
+    SQLTransactionClient* transactionClient() const;
+    SQLTransactionCoordinator* transactionCoordinator() const;
+
 private:
-    Database(Document* document, const String& name, const String& expectedVersion);
+    Database(ScriptExecutionContext* context, const String& name,
+             const String& expectedVersion, const String& displayName,
+             unsigned long estimatedSize);
 
     bool openAndVerifyVersion(ExceptionCode&);
 
     void scheduleTransaction();
     void scheduleTransactionCallback(SQLTransaction*);
-    void scheduleTransactionStep(SQLTransaction* transaction);
-    
-    MessageQueue<RefPtr<SQLTransaction> > m_transactionQueue;
+    void scheduleTransactionStep(SQLTransaction* transaction, bool immediately = false);
+
+    Deque<RefPtr<SQLTransaction> > m_transactionQueue;
     Mutex m_transactionInProgressMutex;
     bool m_transactionInProgress;
+    bool m_isTransactionQueueEnabled;
 
     static void deliverPendingCallback(void*);
 
-    RefPtr<Document> m_document;
-    RefPtr<SecurityOrigin> m_securityOrigin;
+    RefPtr<ScriptExecutionContext> m_scriptExecutionContext;
+    RefPtr<SecurityOrigin> m_mainThreadSecurityOrigin;
+    RefPtr<SecurityOrigin> m_databaseThreadSecurityOrigin;
     String m_name;
     int m_guid;
     String m_expectedVersion;
+    String m_displayName;
+    unsigned long m_estimatedSize;
     String m_filename;
 
     bool m_deleted;
-    
+
     bool m_stopped;
 
     bool m_opened;
@@ -148,7 +163,7 @@ private:
     RefPtr<DatabaseAuthorizer> m_databaseAuthorizer;
 
 #ifndef NDEBUG
-    String databaseDebugName() const { return m_securityOrigin->toString() + "::" + m_name; }
+    String databaseDebugName() const { return m_mainThreadSecurityOrigin->toString() + "::" + m_name; }
 #endif
 };
 
