@@ -97,6 +97,7 @@ bool WebSocket::isAvailable()
 WebSocket::WebSocket(ScriptExecutionContext* context)
     : ActiveDOMObject(context, this)
     , m_state(CONNECTING)
+    , m_bufferedAmountAfterClose(0)
 {
 }
 
@@ -162,9 +163,12 @@ bool WebSocket::send(const String& message, ExceptionCode& ec)
         return false;
     }
     // No exception is raised if the connection was once established but has subsequently been closed.
-    if (m_state == CLOSED)
+    if (m_state == CLOSED) {
+        m_bufferedAmountAfterClose += message.utf8().length() + 2; // 2 for frameing
         return false;
+    }
     // FIXME: check message is valid utf8.
+    ASSERT(m_channel);
     return m_channel->send(message);
 }
 
@@ -174,6 +178,7 @@ void WebSocket::close()
     if (m_state == CLOSED)
         return;
     m_state = CLOSED;
+    m_bufferedAmountAfterClose = m_channel->bufferedAmount();
     m_channel->close();
 }
 
@@ -191,7 +196,7 @@ unsigned long WebSocket::bufferedAmount() const
 {
     if (m_state == OPEN)
         return m_channel->bufferedAmount();
-    return 0;
+    return m_bufferedAmountAfterClose;
 }
 
 ScriptExecutionContext* WebSocket::scriptExecutionContext() const
@@ -223,7 +228,7 @@ void WebSocket::didConnect()
 {
     LOG(Network, "WebSocket %p didConnect", this);
     if (m_state != CONNECTING || !scriptExecutionContext()) {
-        didClose();
+        didClose(0);
         return;
     }
     m_state = OPEN;
@@ -240,10 +245,11 @@ void WebSocket::didReceiveMessage(const String& msg)
     dispatchEvent(evt);
 }
 
-void WebSocket::didClose()
+void WebSocket::didClose(unsigned long unhandledBufferedAmount)
 {
     LOG(Network, "WebSocket %p didClose", this);
     m_state = CLOSED;
+    m_bufferedAmountAfterClose += unhandledBufferedAmount;
     dispatchEvent(Event::create(eventNames().closeEvent, false, false));
     m_channel = 0;
     if (hasPendingActivity())
