@@ -57,7 +57,6 @@ WebSocketChannel::WebSocketChannel(ScriptExecutionContext* context, WebSocketCha
     , m_handshake(url, protocol, context)
     , m_buffer(0)
     , m_bufferSize(0)
-    , m_unhandledBufferSize(0)
 {
 }
 
@@ -78,22 +77,18 @@ void WebSocketChannel::connect()
 bool WebSocketChannel::send(const String& msg)
 {
     LOG(Network, "WebSocketChannel %p send %s", this, msg.utf8().data());
+    ASSERT(m_handle);
     Vector<char> buf;
     buf.append('\0');  // frame type
     buf.append(msg.utf8().data(), msg.utf8().length());
     buf.append('\xff');  // frame end
-    if (!m_handle) {
-        m_unhandledBufferSize += buf.size();
-        return false;
-    }
     return m_handle->send(buf.data(), buf.size());
 }
 
 unsigned long WebSocketChannel::bufferedAmount() const
 {
     LOG(Network, "WebSocketChannel %p bufferedAmount", this);
-    if (!m_handle)
-        return m_unhandledBufferSize;
+    ASSERT(m_handle);
     return m_handle->bufferedAmount();
 }
 
@@ -126,14 +121,14 @@ void WebSocketChannel::didOpen(SocketStreamHandle* handle)
 void WebSocketChannel::didClose(SocketStreamHandle* handle)
 {
     LOG(Network, "WebSocketChannel %p didClose", this);
-    ASSERT(handle == m_handle || !m_handle);
+    ASSERT_UNUSED(handle, handle == m_handle || !m_handle);
     if (m_handle) {
-        m_unhandledBufferSize = handle->bufferedAmount();
+        unsigned long unhandledBufferedAmount = m_handle->bufferedAmount();
         WebSocketChannelClient* client = m_client;
         m_client = 0;
         m_handle = 0;
         if (client)
-            client->didClose();
+            client->didClose(unhandledBufferedAmount);
     }
     deref();
 }
@@ -169,6 +164,8 @@ void WebSocketChannel::didReceiveData(SocketStreamHandle* handle, const char* da
             // FIXME: handle set-cookie2.
             LOG(Network, "WebSocketChannel %p connected", this);
             m_client->didConnect();
+            if (!m_client)
+                return;
             break;
         default:
             LOG(Network, "WebSocketChannel %p connection failed", this);
@@ -214,6 +211,8 @@ void WebSocketChannel::didReceiveData(SocketStreamHandle* handle, const char* da
             if (p < end && *p == '\xff') {
                 if (frameByte == 0x00)
                     m_client->didReceiveMessage(String::fromUTF8(msgStart, p - msgStart));
+                if (!m_client)
+                    return;
                 ++p;
                 nextFrame = p;
             }
