@@ -64,7 +64,11 @@ void ChromeClient::chromeDestroyed()
 FloatRect ChromeClient::windowRect()
 {
     GtkWidget* window = gtk_widget_get_toplevel(GTK_WIDGET(m_webView));
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (gtk_widget_is_toplevel(window)) {
+#else
     if (GTK_WIDGET_TOPLEVEL(window)) {
+#endif
         gint left, top, width, height;
         gtk_window_get_position(GTK_WINDOW(window), &left, &top);
         gtk_window_get_size(GTK_WINDOW(window), &width, &height);
@@ -84,6 +88,23 @@ void ChromeClient::setWindowRect(const FloatRect& rect)
                  "width", intrect.width(),
                  "height", intrect.height(),
                  NULL);
+
+    gboolean autoResizeWindow;
+    WebKitWebSettings* settings = webkit_web_view_get_settings(m_webView);
+    g_object_get(settings, "auto-resize-window", &autoResizeWindow, NULL);
+
+    if (!autoResizeWindow)
+        return;
+
+    GtkWidget* window = gtk_widget_get_toplevel(GTK_WIDGET(m_webView));
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (gtk_widget_is_toplevel(window)) {
+#else
+    if (GTK_WIDGET_TOPLEVEL(window)) {
+#endif
+        gtk_window_move(GTK_WINDOW(window), intrect.x(), intrect.y());
+        gtk_window_resize(GTK_WINDOW(window), intrect.width(), intrect.height());
+    }
 }
 
 FloatRect ChromeClient::pageRect()
@@ -106,7 +127,11 @@ void ChromeClient::focus()
 void ChromeClient::unfocus()
 {
     GtkWidget* window = gtk_widget_get_toplevel(GTK_WIDGET(m_webView));
+#if GTK_CHECK_VERSION(2, 18, 0)
+    if (gtk_widget_is_toplevel(window))
+#else
     if (GTK_WIDGET_TOPLEVEL(window))
+#endif
         gtk_window_set_focus(GTK_WINDOW(window), NULL);
 }
 
@@ -234,7 +259,11 @@ void ChromeClient::closeWindowSoon()
 
 bool ChromeClient::canTakeFocus(FocusDirection)
 {
+#if GTK_CHECK_VERSION(2, 18, 0)
+    return gtk_widget_get_can_focus(GTK_WIDGET(m_webView));
+#else
     return GTK_WIDGET_CAN_FOCUS(m_webView);
+#endif
 }
 
 void ChromeClient::takeFocus(FocusDirection)
@@ -242,6 +271,10 @@ void ChromeClient::takeFocus(FocusDirection)
     unfocus();
 }
 
+void ChromeClient::focusedNodeChanged(Node*)
+{
+}
+
 bool ChromeClient::canRunBeforeUnloadConfirmPanel()
 {
     return true;
@@ -438,20 +471,6 @@ void ChromeClient::scrollbarsModeDidChange() const
 
 void ChromeClient::mouseDidMoveOverElement(const HitTestResult& hit, unsigned modifierFlags)
 {
-    // If a tooltip must be displayed it will be, afterwards, when
-    // setToolTip is called; this is just a work-around to make sure
-    // it updates its location correctly; see
-    // https://bugs.webkit.org/show_bug.cgi?id=15793.
-    g_object_set(m_webView, "has-tooltip", FALSE, NULL);
-
-    GdkDisplay* gdkDisplay;
-    GtkWidget* window = gtk_widget_get_toplevel(GTK_WIDGET(m_webView));
-    if (GTK_WIDGET_TOPLEVEL(window))
-        gdkDisplay = gtk_widget_get_display(window);
-    else
-        gdkDisplay = gdk_display_get_default();
-    gtk_tooltip_trigger_tooltip_query(gdkDisplay);
-
     // check if the element is a link...
     bool isLink = hit.isLiveLink();
     if (isLink) {
@@ -471,16 +490,7 @@ void ChromeClient::mouseDidMoveOverElement(const HitTestResult& hit, unsigned mo
 
 void ChromeClient::setToolTip(const String& toolTip, TextDirection)
 {
-#if GTK_CHECK_VERSION(2,12,0)
-    if (toolTip.isEmpty())
-        g_object_set(m_webView, "has-tooltip", FALSE, NULL);
-    else
-        gtk_widget_set_tooltip_text(GTK_WIDGET(m_webView), toolTip.utf8().data());
-#else
-    // TODO: Support older GTK+ versions
-    // See http://bugs.webkit.org/show_bug.cgi?id=15793
-    notImplemented();
-#endif
+    webkit_web_view_set_tooltip_text(m_webView, toolTip.utf8().data());
 }
 
 void ChromeClient::print(Frame* frame)
@@ -553,6 +563,12 @@ void ChromeClient::runOpenPanel(Frame*, PassRefPtr<FileChooser> prpFileChooser)
     gtk_widget_destroy(dialog);
 }
 
+void ChromeClient::iconForFiles(const Vector<WebCore::String>&, PassRefPtr<WebCore::FileChooser>)
+{
+    // FIXME: Move the code in Icon::createIconForFiles() here.
+    notImplemented();
+}
+
 bool ChromeClient::setCursor(PlatformCursorHandle)
 {
     notImplemented();
