@@ -47,6 +47,8 @@ namespace WebKit {
         virtual bool canTakeFocus(WebCore::FocusDirection);
         virtual void takeFocus(WebCore::FocusDirection);
 
+        virtual void focusedNodeChanged(WebCore::Node*);
+
         virtual WebCore::Page* createWindow(WebCore::Frame*, const WebCore::FrameLoadRequest&, const WebCore::WindowFeatures&);
         virtual void show();
 
@@ -105,6 +107,7 @@ namespace WebKit {
         virtual void reachedMaxAppCacheSize(int64_t spaceNeeded);
 #endif
         virtual void runOpenPanel(WebCore::Frame*, PassRefPtr<WebCore::FileChooser>);
+        virtual void iconForFiles(const Vector<WebCore::String>&, PassRefPtr<WebCore::FileChooser>);
 
         virtual void formStateDidChange(const WebCore::Node*) { }
 
