@@ -22,6 +22,7 @@
 
 #include "CString.h"
 #include <glib/gi18n-lib.h>
+#include "GRefPtr.h"
 #include "Noncopyable.h"
 #include "NotImplemented.h"
 #include "ResourceHandleClient.h"
@@ -69,7 +70,7 @@ class DownloadClient : public Noncopyable, public ResourceHandleClient {
 struct _WebKitDownloadPrivate {
     gchar* destinationURI;
     gchar* suggestedFilename;
-    guint currentSize;
+    guint64 currentSize;
     GTimer* timer;
     WebKitDownloadStatus status;
     GFileOutputStream* outputStream;
@@ -227,10 +228,14 @@ static void webkit_download_class_init(WebKitDownloadClass* downloadClass)
     /**
      * WebKitDownload::error:
      * @download: the object on which the signal is emitted
-     * @current_bytes: the current count of bytes downloaded
-     * @total_bytes: the total bytes count in the downloaded file, aka file size.
+     * @error_code: the corresponding error code
+     * @error_detail: detailed error code for the error, see
+     * #WebKitDownloadError
+     * @reason: a string describing the error
      *
-     * Indicates an error in the download.
+     * Emitted when @download is interrupted either by user action or by
+     * network errors, @error_detail will take any value of
+     * #WebKitDownloadError.
      *
      * Since: 1.1.2
      */
@@ -408,7 +413,8 @@ WebKitDownload* webkit_download_new_with_handle(WebKitNetworkRequest* request, W
     g_return_val_if_fail(request, NULL);
 
     ResourceHandleInternal* d = handle->getInternal();
-    soup_session_pause_message(webkit_get_default_session(), d->m_msg);
+    if (d->m_msg)
+        soup_session_pause_message(webkit_get_default_session(), d->m_msg);
 
     WebKitDownload* download = WEBKIT_DOWNLOAD(g_object_new(WEBKIT_TYPE_DOWNLOAD, "network-request", request, NULL));
     WebKitDownloadPrivate* priv = download->priv;
@@ -479,7 +485,8 @@ void webkit_download_start(WebKitDownload* download)
         priv->resourceHandle->setClient(priv->downloadClient);
 
         ResourceHandleInternal* d = priv->resourceHandle->getInternal();
-        soup_session_unpause_message(webkit_get_default_session(), d->m_msg);
+        if (d->m_msg)
+            soup_session_unpause_message(webkit_get_default_session(), d->m_msg);
     }
 
     priv->timer = g_timer_new();
@@ -754,7 +761,7 @@ guint64 webkit_download_get_total_size(WebKitDownload* download)
     if (!message)
         return 0;
 
-    return MAX(priv->currentSize, soup_message_headers_get_content_length(message->response_headers));
+    return MAX(priv->currentSize, static_cast<guint64>(soup_message_headers_get_content_length(message->response_headers)));
 }
 
 /**
@@ -853,22 +860,24 @@ static void webkit_download_received_data(WebKitDownload* download, const gchar*
     if (priv->currentSize > webkit_download_get_total_size(download))
         g_object_notify(G_OBJECT(download), "total-size");
 
-    gdouble lastProgress = webkit_download_get_progress(download);
-
     // Throttle progress notification to not consume high amounts of
-    // CPU on fast links, except when the progress is >= 3%, or we
-    // reached the end.
+    // CPU on fast links, except when the last notification occured
+    // in more then 0.7 secs from now, or the last notified progress
+    // is passed in 1% or we reached the end.
+    static gdouble lastProgress = 0;
     static gdouble lastElapsed = 0;
     gdouble currentElapsed = g_timer_elapsed(priv->timer, NULL);
+    gdouble currentProgress = webkit_download_get_progress(download);
 
     if (lastElapsed
-        && (currentElapsed - lastElapsed) < 0.1
-        && (webkit_download_get_progress(download) - lastProgress) < 0.03
-        && webkit_download_get_progress(download) < 1.0) {
-        lastElapsed = currentElapsed;
+        && lastProgress
+        && (currentElapsed - lastElapsed) < 0.7
+        && (currentProgress - lastProgress) < 0.01
+        && currentProgress < 1.0) {
         return;
     }
     lastElapsed = currentElapsed;
+    lastProgress = currentProgress;
 
     g_object_notify(G_OBJECT(download), "progress");
 }
@@ -890,6 +899,7 @@ static void webkit_download_error(WebKitDownload* download, const ResourceError&
     webkit_download_close_stream(download);
 
     WebKitDownloadPrivate* priv = download->priv;
+    GRefPtr<WebKitDownload> protect(download);
 
     g_timer_stop(priv->timer);
     webkit_download_set_status(download, WEBKIT_DOWNLOAD_STATUS_ERROR);
