@@ -4,6 +4,8 @@
  * Copyright (C) 2008 Collabora Ltd.
  * Copyright (C) 2008 Holger Hans Peter Freyther
  * Copyright (C) 2009 Jan Michael Alonzo
+ * Copyright (C) 2009 Movial Creative Technologies Inc.
+ * Copyright (C) 2009 Igalia S.L.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -35,7 +37,7 @@
 #include "PlatformString.h"
 
 #include <glib/gi18n-lib.h>
-#if PLATFORM(UNIX)
+#if OS(UNIX)
 #include <sys/utsname.h>
 #endif
 
@@ -96,8 +98,14 @@ struct _WebKitWebSettingsPrivate {
     gboolean enable_offline_web_application_cache;
     WebKitEditingBehavior editing_behavior;
     gboolean enable_universal_access_from_file_uris;
-    gboolean enable_web_sockets;
+    gboolean enable_file_access_from_file_uris;
     gboolean enable_dom_paste;
+    gboolean tab_key_cycles_through_elements;
+    gboolean enable_default_context_menu;
+    gboolean enable_site_specific_quirks;
+    gboolean enable_page_cache;
+    gboolean auto_resize_window;
+    gboolean enable_java_applet;
 };
 
 #define WEBKIT_WEB_SETTINGS_GET_PRIVATE(obj) (G_TYPE_INSTANCE_GET_PRIVATE((obj), WEBKIT_TYPE_WEB_SETTINGS, WebKitWebSettingsPrivate))
@@ -138,8 +146,14 @@ enum {
     PROP_ENABLE_OFFLINE_WEB_APPLICATION_CACHE,
     PROP_EDITING_BEHAVIOR,
     PROP_ENABLE_UNIVERSAL_ACCESS_FROM_FILE_URIS,
-    PROP_ENABLE_WEB_SOCKETS,
-    PROP_ENABLE_DOM_PASTE
+    PROP_ENABLE_FILE_ACCESS_FROM_FILE_URIS,
+    PROP_ENABLE_DOM_PASTE,
+    PROP_TAB_KEY_CYCLES_THROUGH_ELEMENTS,
+    PROP_ENABLE_DEFAULT_CONTEXT_MENU,
+    PROP_ENABLE_SITE_SPECIFIC_QUIRKS,
+    PROP_ENABLE_PAGE_CACHE,
+    PROP_AUTO_RESIZE_WINDOW,
+    PROP_ENABLE_JAVA_APPLET
 };
 
 // Create a default user agent string
@@ -152,7 +166,7 @@ static String webkit_get_user_agent()
 
 #if PLATFORM(X11)
     platform = g_strdup("X11");
-#elif PLATFORM(WIN_OS)
+#elif OS(WINDOWS)
     platform = g_strdup("Windows");
 #elif PLATFORM(MAC)
     platform = g_strdup("Macintosh");
@@ -163,22 +177,22 @@ static String webkit_get_user_agent()
 #endif
 
    // FIXME: platform/version detection can be shared.
-#if PLATFORM(DARWIN)
+#if OS(DARWIN)
 
-#if PLATFORM(X86)
+#if CPU(X86)
     osVersion = g_strdup("Intel Mac OS X");
 #else
     osVersion = g_strdup("PPC Mac OS X");
 #endif
 
-#elif PLATFORM(UNIX)
+#elif OS(UNIX)
     struct utsname name;
     if (uname(&name) != -1)
         osVersion = g_strdup_printf("%s %s", name.sysname, name.machine);
     else
         osVersion = g_strdup("Unknown");
 
-#elif PLATFORM(WIN_OS)
+#elif OS(WINDOWS)
     // FIXME: Compute the Windows version
     osVersion = g_strdup("Windows");
 
@@ -651,24 +665,6 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          FALSE,
                                                          flags));
 
-    /**
-     * WebKitWebSettings:enable-web-sockets
-     *
-     * Whether to enable support for Web Sockets.
-     *
-     * Implementation of Web Sockets is currently considered experimental.
-     * Name of this property and the behavior could change in the future.
-     *
-     * Since: 1.1.16
-     */
-    g_object_class_install_property(gobject_class,
-                                    PROP_ENABLE_WEB_SOCKETS,
-                                    g_param_spec_boolean("enable-web-sockets",
-                                                         _("Enable Web Sockets"),
-                                                         _("Whether to enable Web Sockets"),
-                                                         FALSE,
-                                                         flags));
-
     /**
      * WebKitWebSettings:enable-dom-paste
      *
@@ -684,6 +680,144 @@ static void webkit_web_settings_class_init(WebKitWebSettingsClass* klass)
                                                          _("Whether to enable DOM paste"),
                                                          FALSE,
                                                          flags));
+    /**
+    * WebKitWebSettings:tab-key-cycles-through-elements:
+    *
+    * Whether the tab key cycles through elements on the page.
+    *
+    * If @flag is %TRUE, pressing the tab key will focus the next element in
+    * the @web_view. If @flag is %FALSE, the @web_view will interpret tab
+    * key presses as normal key presses. If the selected element is editable, the
+    * tab key will cause the insertion of a tab character.
+    *
+    * Since: 1.1.17
+    */
+    g_object_class_install_property(gobject_class,
+                                    PROP_TAB_KEY_CYCLES_THROUGH_ELEMENTS,
+                                    g_param_spec_boolean("tab-key-cycles-through-elements",
+                                                         _("Tab key cycles through elements"),
+                                                         _("Whether the tab key cycles through elements on the page."),
+                                                         TRUE,
+                                                         flags));
+
+    /**
+     * WebKitWebSettings:enable-default-context-menu:
+     *
+     * Whether right-clicks should be handled automatically to create,
+     * and display the context menu. Turning this off will make
+     * WebKitGTK+ not emit the populate-popup signal. Notice that the
+     * default button press event handler may still handle right
+     * clicks for other reasons, such as in-page context menus, or
+     * right-clicks that are handled by the page itself.
+     *
+     * Since: 1.1.18
+     */
+    g_object_class_install_property(gobject_class,
+                                    PROP_ENABLE_DEFAULT_CONTEXT_MENU,
+                                    g_param_spec_boolean(
+                                    "enable-default-context-menu",
+                                    _("Enable Default Context Menu"),
+                                    _("Enables the handling of right-clicks for the creation of the default context menu"),
+                                    TRUE,
+                                    flags));
+
+    /**
+     * WebKitWebSettings::enable-site-specific-quirks
+     *
+     * Whether to turn on site-specific hacks.  Turning this on will
+     * tell WebKitGTK+ to use some site-specific workarounds for
+     * better web compatibility.  For example, older versions of
+     * MediaWiki will incorrectly send WebKit a css file with KHTML
+     * workarounds.  By turning on site-specific quirks, WebKit will
+     * special-case this and other cases to make the sites work.
+     *
+     * Since: 1.1.18
+     */
+    g_object_class_install_property(gobject_class,
+                                    PROP_ENABLE_SITE_SPECIFIC_QUIRKS,
+                                    g_param_spec_boolean(
+                                    "enable-site-specific-quirks",
+                                    _("Enable Site Specific Quirks"),
+                                    _("Enables the site-specific compatibility workarounds"),
+                                    FALSE,
+                                    flags));
+
+    /**
+    * WebKitWebSettings:enable-page-cache:
+    *
+    * Enable or disable the page cache. Disabling the page cache is
+    * generally only useful for special circumstances like low-memory
+    * scenarios or special purpose applications like static HTML
+    * viewers. This setting only controls the Page Cache, this cache
+    * is different than the disk-based or memory-based traditional
+    * resource caches, its point is to make going back and forth
+    * between pages much faster. For details about the different types
+    * of caches and their purposes see:
+    * http://webkit.org/blog/427/webkit-page-cache-i-the-basics/
+    *
+    * Since: 1.1.18
+    */
+    g_object_class_install_property(gobject_class,
+                                    PROP_ENABLE_PAGE_CACHE,
+                                    g_param_spec_boolean("enable-page-cache",
+                                                         _("Enable page cache"),
+                                                         _("Whether the page cache should be used"),
+                                                         FALSE,
+                                                         flags));
+
+    /**
+    * WebKitWebSettings:auto-resize-window:
+    *
+    * Web pages can request to modify the size and position of the
+    * window containing the #WebKitWebView through various DOM methods
+    * (resizeTo, moveTo, resizeBy, moveBy). By default WebKit will not
+    * honor this requests, but you can set this property to %TRUE if
+    * you'd like it to do so. If you wish to handle this manually, you
+    * can connect to the notify signal for the
+    * #WebKitWebWindowFeatures of your #WebKitWebView.
+    * 
+    * Since: 1.1.22
+    */
+    g_object_class_install_property(gobject_class,
+                                    PROP_AUTO_RESIZE_WINDOW,
+                                    g_param_spec_boolean("auto-resize-window",
+                                                         _("Auto Resize Window"),
+                                                         _("Automatically resize the toplevel window when a page requests it"),
+                                                         FALSE,
+                                                         flags));
+
+    /**
+     * WebKitWebSettings:enable-file-access-from-file-uris:
+     *
+     * Boolean property to control file access for file:// URIs. If this
+     * option is enabled every file:// will have its own security unique domain.
+     *
+     * Since: 1.1.22
+     */
+     g_object_class_install_property(gobject_class,
+                                     PROP_ENABLE_FILE_ACCESS_FROM_FILE_URIS,
+                                     g_param_spec_boolean("enable-file-access-from-file-uris",
+                                                          "Enable file access from file URIs",
+                                                          "Controls file access for file:// URIs.",
+                                                          FALSE,
+                                                          flags));
+
+   /**
+    * WebKitWebSettings:enable-java-applet:
+    *
+    * Enable or disable support for the Java <applet> tag. Keep in
+    * mind that Java content can be still shown in the page through
+    * <object> or <embed>, which are the preferred tags for this task.
+    *
+    * Since: 1.1.22
+    */
+    g_object_class_install_property(gobject_class,
+                                    PROP_ENABLE_JAVA_APPLET,
+                                    g_param_spec_boolean("enable-java-applet",
+                                                         _("Enable Java Applet"),
+                                                         _("Whether Java Applet support through <applet> should be enabled"),
+                                                         TRUE,
+                                                         flags));
 
     g_type_class_add_private(klass, sizeof(WebKitWebSettingsPrivate));
 }
@@ -878,12 +1012,30 @@ static void webkit_web_settings_set_property(GObject* object, guint prop_id, con
     case PROP_ENABLE_UNIVERSAL_ACCESS_FROM_FILE_URIS:
         priv->enable_universal_access_from_file_uris = g_value_get_boolean(value);
         break;
-    case PROP_ENABLE_WEB_SOCKETS:
-        priv->enable_web_sockets = g_value_get_boolean(value);
+    case PROP_ENABLE_FILE_ACCESS_FROM_FILE_URIS:
+        priv->enable_file_access_from_file_uris = g_value_get_boolean(value);
         break;
     case PROP_ENABLE_DOM_PASTE:
         priv->enable_dom_paste = g_value_get_boolean(value);
         break;
+    case PROP_TAB_KEY_CYCLES_THROUGH_ELEMENTS:
+        priv->tab_key_cycles_through_elements = g_value_get_boolean(value);
+        break;
+    case PROP_ENABLE_DEFAULT_CONTEXT_MENU:
+        priv->enable_default_context_menu = g_value_get_boolean(value);
+        break;
+    case PROP_ENABLE_SITE_SPECIFIC_QUIRKS:
+        priv->enable_site_specific_quirks = g_value_get_boolean(value);
+        break;
+    case PROP_ENABLE_PAGE_CACHE:
+        priv->enable_page_cache = g_value_get_boolean(value);
+        break;
+    case PROP_AUTO_RESIZE_WINDOW:
+        priv->auto_resize_window = g_value_get_boolean(value);
+        break;
+    case PROP_ENABLE_JAVA_APPLET:
+        priv->enable_java_applet = g_value_get_boolean(value);
+        break;
     default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
@@ -986,22 +1138,40 @@ static void webkit_web_settings_get_property(GObject* object, guint prop_id, GVa
     case PROP_JAVASCRIPT_CAN_OPEN_WINDOWS_AUTOMATICALLY:
         g_value_set_boolean(value, priv->javascript_can_open_windows_automatically);
         break;
-   case PROP_ENABLE_OFFLINE_WEB_APPLICATION_CACHE:
+    case PROP_ENABLE_OFFLINE_WEB_APPLICATION_CACHE:
         g_value_set_boolean(value, priv->enable_offline_web_application_cache);
         break;
     case PROP_EDITING_BEHAVIOR:
         g_value_set_enum(value, priv->editing_behavior);
         break;
-   case PROP_ENABLE_UNIVERSAL_ACCESS_FROM_FILE_URIS:
+    case PROP_ENABLE_UNIVERSAL_ACCESS_FROM_FILE_URIS:
         g_value_set_boolean(value, priv->enable_universal_access_from_file_uris);
         break;
-    case PROP_ENABLE_WEB_SOCKETS:
-        g_value_set_boolean(value, priv->enable_web_sockets);
+    case PROP_ENABLE_FILE_ACCESS_FROM_FILE_URIS:
+        g_value_set_boolean(value, priv->enable_file_access_from_file_uris);
         break;
     case PROP_ENABLE_DOM_PASTE:
         g_value_set_boolean(value, priv->enable_dom_paste);
         break;
-   default:
+    case PROP_TAB_KEY_CYCLES_THROUGH_ELEMENTS:
+        g_value_set_boolean(value, priv->tab_key_cycles_through_elements);
+        break;
+    case PROP_ENABLE_DEFAULT_CONTEXT_MENU:
+        g_value_set_boolean(value, priv->enable_default_context_menu);
+        break;
+    case PROP_ENABLE_SITE_SPECIFIC_QUIRKS:
+        g_value_set_boolean(value, priv->enable_site_specific_quirks);
+        break;
+    case PROP_ENABLE_PAGE_CACHE:
+        g_value_set_boolean(value, priv->enable_page_cache);
+        break;
+    case PROP_AUTO_RESIZE_WINDOW:
+        g_value_set_boolean(value, priv->auto_resize_window);
+        break;
+    case PROP_ENABLE_JAVA_APPLET:
+        g_value_set_boolean(value, priv->enable_java_applet);
+        break;
+    default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
         break;
     }
@@ -1065,8 +1235,14 @@ WebKitWebSettings* webkit_web_settings_copy(WebKitWebSettings* web_settings)
                  "enable-offline-web-application-cache", priv->enable_offline_web_application_cache,
                  "editing-behavior", priv->editing_behavior,
                  "enable-universal-access-from-file-uris", priv->enable_universal_access_from_file_uris,
-                 "enable-web-sockets", priv->enable_web_sockets,
+                 "enable-file-access-from-file-uris", priv->enable_file_access_from_file_uris,
                  "enable-dom-paste", priv->enable_dom_paste,
+                 "tab-key-cycles-through-elements", priv->tab_key_cycles_through_elements,
+                 "enable-default-context-menu", priv->enable_default_context_menu,
+                 "enable-site-specific-quirks", priv->enable_site_specific_quirks,
+                 "enable-page-cache", priv->enable_page_cache,
+                 "auto-resize-window", priv->auto_resize_window,
+                 "enable-java-applet", priv->enable_java_applet,
                  NULL));
 
     return copy;
