@@ -56,6 +56,7 @@ public:
     bool isCommandEnabled(JSStringRef name);
     void keepWebHistory();
     void notifyDone();
+    int numberOfPages(float pageWidthInPixels, float pageHeightInPixels);
     void overridePreference(JSStringRef key, JSStringRef value);
     int pageNumberForElementById(JSStringRef id, float pageWidthInPixels, float pageHeightInPixels);
     JSStringRef pathToLocalResource(JSContextRef, JSStringRef url);
@@ -68,6 +69,7 @@ public:
     void removeAllVisitedLinks();
     void setAcceptsEditing(bool acceptsEditing);
     void setAllowUniversalAccessFromFileURLs(bool);
+    void setAllowFileAccessFromFileURLs(bool);
     void setAppCacheMaximumSize(unsigned long long quota);
     void setAuthorAndUserStylesEnabled(bool);
     void setCacheModel(int);
@@ -89,6 +91,7 @@ public:
     void setUserStyleSheetEnabled(bool flag);
     void setUserStyleSheetLocation(JSStringRef path);
     void setXSSAuditorEnabled(bool flag);
+    void setFrameSetFlatteningEnabled(bool enable);
 
     void waitForPolicyDelegate();
     size_t webHistoryItemCount();
@@ -230,6 +233,13 @@ public:
     void evaluateScriptInIsolatedWorld(unsigned worldId, JSObjectRef globalObject, JSStringRef script);
 
     void setPOSIXLocale(JSStringRef locale);
+    
+    // The following API test functions should probably be moved to platform-specific 
+    // unit tests outside of DRT once they exist.
+    void apiTestNewWindowDataLoadBaseURL(JSStringRef utf8Data, JSStringRef baseURL);
+
+    static const unsigned maxViewWidth;
+    static const unsigned maxViewHeight;
 
 private:
     bool m_dumpAsPDF;
