@@ -149,6 +149,12 @@ int LayoutTestController::pageNumberForElementById(JSStringRef id, float pageWid
     return pageNumber;
 }
 
+int LayoutTestController::numberOfPages(float, float)
+{
+    // FIXME: implement
+    return -1;
+}
+
 size_t LayoutTestController::webHistoryItemCount()
 {
     // FIXME: implement
@@ -328,6 +334,11 @@ void LayoutTestController::setXSSAuditorEnabled(bool flag)
     g_object_set(G_OBJECT(settings), "enable-xss-auditor", flag, NULL);
 }
 
+void LayoutTestController::setFrameSetFlatteningEnabled(bool flag)
+{
+    // FIXME: implement
+}
+
 void LayoutTestController::setAllowUniversalAccessFromFileURLs(bool flag)
 {
     WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
@@ -337,6 +348,15 @@ void LayoutTestController::setAllowUniversalAccessFromFileURLs(bool flag)
     g_object_set(G_OBJECT(settings), "enable-universal-access-from-file-uris", flag, NULL);
 }
 
+void LayoutTestController::setAllowFileAccessFromFileURLs(bool flag)
+{
+    WebKitWebView* view = webkit_web_frame_get_web_view(mainFrame);
+    ASSERT(view);
+
+    WebKitWebSettings* settings = webkit_web_view_get_settings(view);
+    g_object_set(G_OBJECT(settings), "enable-file-access-from-file-uris", flag, NULL);
+}
+
 void LayoutTestController::setAuthorAndUserStylesEnabled(bool flag)
 {
     // FIXME: implement
@@ -575,3 +595,8 @@ void LayoutTestController::removeAllVisitedLinks()
 {
     // FIXME: Implement this.
 }
+
+void LayoutTestController::apiTestNewWindowDataLoadBaseURL(JSStringRef utf8Data, JSStringRef baseURL)
+{
+
+}
